package gssim.db.mysql;

import gssim.db.dao.data.ApplicationProperty;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class ApplicationPropertyDAO extends AbstractDAO implements
		gssim.db.dao.ApplicationPropertyDAO {

	protected ApplicationPropertyDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(ApplicationPropertyDAO.class);
		setRelationName("application_property");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		return null;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM application_property ap WHERE"
				+ " (ap.key = ?) AND "
				+ " ap.version = (SELECT MAX(ap2.version) "
				+ " FROM application_property ap2 GROUP BY ap2.key "
				+ " HAVING (ap2.key = ?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO application_property(key, value) VALUES(?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM application_property WHERE version = ? AND key = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		return null;
	}

	@Override
	public boolean delete(ApplicationProperty obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public void fillObjectFromResultSet(ApplicationProperty obj, ResultSet result)
			throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt,
			ApplicationProperty obj, int offset) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public List<ApplicationProperty> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<ApplicationProperty> list = new ArrayList<ApplicationProperty>();

		if (finder.getString("key") == null) throw new SQLException("Key cannot be null");
		
		stmt.setString(1, finder.getString("key"));
		stmt.setString(2, finder.getString("key"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			ApplicationProperty obj = new ApplicationProperty();

			Integer version = result.getIntObj("version");
			String key = result.getString("key");
			String value = result.getString("value");

			obj.setVersion(version);
			obj.setKey(key);
			obj.setValue(value);

			list.add(obj);
		}

		return list;
	}

	@Override
	public int insert(ApplicationProperty obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		stmt.setString(1, obj.getKey());
		stmt.setString(2, obj.getValue());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("insert");
		} else
			logError("insert", result, obj);

		return result;
	}

	@Override
	public int[] insertBatch(List<ApplicationProperty> objList, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public ApplicationProperty select(ApplicationProperty obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getVersion());
		stmt.setString(2, obj.getKey());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			obj.setValue(result.getString("value"));
			return obj;
		}

		// no reservation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");
	}

	@Override
	public boolean update(ApplicationProperty obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

}