package gssim.db.mysql;

import gssim.db.dao.data.UnitDetails;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class UnitDetailsDAO extends AbstractDAO implements
		gssim.db.dao.UnitDetailsDAO {

	protected UnitDetailsDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(UnitDetailsDAO.class);
		setRelationName("unit_details");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		return null;
	}

	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM unit_details WHERE"
				+ " (fk_unit_id = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO unit_details VALUES(?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM unit_details WHERE fk_unit_id = ? AND name = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		return null;
	}

	@Override
	public boolean delete(UnitDetails obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public void fillObjectFromResultSet(UnitDetails obj, ResultSet result)
			throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt, UnitDetails obj,
			int offset) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public List<UnitDetails> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<UnitDetails> list = new ArrayList<UnitDetails>();

		stmt.setInt(1, finder.getInteger("fkUnitId"));
		stmt.setInt(2, finder.getInteger("fkUnitId"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			UnitDetails obj = new UnitDetails();

			Integer fkUnitId = result.getIntObj("fk_unit_id");
			String name = result.getString("name");
			String value = result.getString("value");

			obj.setFkUnitId(fkUnitId);
			obj.setName(name);
			obj.setValue(value);

			list.add(obj);
		}

		return list;
	}

	@Override
	public int insert(UnitDetails obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		stmt.setInt(1, obj.getFkUnitId());
		stmt.setString(2, obj.getName());
		stmt.setString(3, obj.getValue());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("insert");
		} else
			logError("insert", result, obj);

		return result;
	}

	@Override
	public int[] insertBatch(List<UnitDetails> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			stmt.setInt(1, objList.get(i).getFkUnitId());
			stmt.setString(2, objList.get(i).getName());
			stmt.setString(3, objList.get(i).getValue());
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {
			if (commit)
				commit();

			logSuccess("batch insert", j);

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public UnitDetails select(UnitDetails obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getFkUnitId());
		stmt.setString(2, obj.getName());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			obj.setValue(result.getString("value"));
			return obj;
		}

		// no reservation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");
	}

	@Override
	public boolean update(UnitDetails obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

}